// This file's extension implies that it's C, but it's really -*- C++ -*-.
/*
 * Copyright (C) 2002-2022 CERN for the benefit of the ATLAS collaboration.
 */
/**
 * @file GaudiKernel/BatchedAlgorithm.h
 * @author scott snyder <snyder@bnl.gov>
 * @date Aug, 2022
 * @brief 
 */


#ifndef GAUDIKERNEL_BATCHEDALGORITHM_H
#define GAUDIKERNEL_BATCHEDALGORITHM_H

#include "Gaudi/Algorithm.h"
#include "GaudiKernel/ThreadLocalContext.h"


namespace Gaudi {


class BatchedAlgorithm
  : public Algorithm
{
public:
  using Algorithm::Algorithm;

  // FIXME TODO: Bundle this with iteration over the ctx vector?
  class WithContext
  {
  public:
    WithContext (const BatchedAlgorithm* algo, const EventContext& ctx)
      : m_algo (algo),
        m_savedCtx ( Gaudi::Hive::currentContext() )
    {
      Gaudi::Hive::setCurrentContext( ctx );
      m_algo->whiteboard()->selectStore( ctx.valid() ? ctx.slot() : 0 ).ignore();
    }
    ~WithContext()
    {
      Gaudi::Hive::setCurrentContext( m_savedCtx );
      m_algo->whiteboard()->selectStore( m_savedCtx.valid() ? m_savedCtx.slot() : 0 ).ignore();
    }
  private:
    const BatchedAlgorithm* m_algo;
    EventContext m_savedCtx;
  };

  // FIXME TODO: Suppress warnings about execute being hidden like was done
  // for the legacy Algorithm.

  virtual StatusCode execute( const EventContext& ) const override final;
  virtual StatusCode execute( const std::vector<const EventContext*>& ) const = 0;

  // FIXME TODO: This should be defined in the Algorithm base class.
  // Not doing it yet to avoid having to recompile everything.
  virtual unsigned int batchSize() const { return m_batchSize; }

  virtual StatusCode sysExecute( const EventContext& ctx ) override final;
  StatusCode sysExecute( const std::vector<EventContext*>& );

private:
  Gaudi::Property<int> m_batchSize
  { this, "BatchSize", 1,
      "How many events to process at once - 0 means treat as unbatched" };
};


} // namespace Gaudi


#endif // not GAUDIKERNEL_BATCHEDALGORITHM_H
